/* vim: set ts=4 sw=4 et */
/******************************************************
 * Inline (Google) Definitions Firefox Extension
 ******************************************************
 * version 1.0
 * 2009-06-13
 *  Firefox 3.5 support
 *  Proper namespacing
 *  Menu items for back/close
 *  Misc tweaks
 *
 *
 *
 *
 * version 0.5
 * 2006-10-04
 *  Firefox 2 support
 *  Back button for nested lookups

 * version 0.4
 * 2006-02-17
 *  by Eug

 * based in large part on the Greasemonkey script
 * by Brent Charbonneau:
 *  Wikipedia Inline Article Viewer
 *  version 0.1.2
 *  2005-09-03
 *  Copyright (c) 2005, Brent Charbonneau
 *  Released under the GPL license
 *  http://www.gnu.org/copyleft/gpl.html
 *
 * Also using a bit of code from "Dictionary Search"
 * by Jaap Haitsma

 * ...this is also released under the GPL :)
 */

/* namespace - everything will be in this one global variable */
var InlineDefinitionsExtension = {};
(function() {


this.log = function (str) {
    dump(str + "\n");
}

this.dumpObject = function(obj) {
    /*
    dump("\n dumping object contents\n");
    for(i in obj){
        dump(i + " = " + obj[i] + "\n");
    }
    dump("\n");
    */
}

var popupLeft, popupTop, popupClientTop;
 

this.init = function () {
    InlineDefinitionsExtension.log("inlineDefinitionsInit()");

    /* Register context menu event */
    document.getElementById("contentAreaContextMenu").addEventListener("popupshowing", InlineDefinitionsExtension.onPopupShowing, false);

    /* Add our override stylesheet for websites that mess with <li> and <ul> */
    var sss = Components.classes["@mozilla.org/content/style-sheet-service;1"] .getService(Components.interfaces.nsIStyleSheetService);
    var ios = Components.classes["@mozilla.org/network/io-service;1"] .getService(Components.interfaces.nsIIOService);
    var uri = ios.newURI("chrome://inlinedefinitions/content/override.css", null, null);
    if(!sss.sheetRegistered(uri, sss.AUTHOR_SHEET)) {
        sss.loadAndRegisterSheet(uri, sss.AUTHOR_SHEET);
    }
}

/* hide the Inline Search item when apropriate (use same logic as for web search) */
this.onPopupShowing = function (event) { 
    var selection = InlineDefinitionsExtension.getSelection();
    var selectedText = selection.join(" ");   
    var menuItem1 = document.getElementById("context_inlinedefinitions");
    var menuItem2 = document.getElementById("context_inlinedefinitions_close");
    var menuItem3 = document.getElementById("context_inlinedefinitions_back");
       
    /* Main menu item */
    if (selectedText != ""){
        menuItem1.hidden = false;        
        menuItem1.setAttribute("label", "Inline Definitions");
    }
    else{
        menuItem1.hidden = true;
    }

    /* back & close menu items */
    if (InlineDefinitionsExtension.isInsideDefinitionsWindow(document.popupNode)) {
        menuItem2.hidden = false;
        menuItem2.setAttribute("label", "Close Inline Definitions");

        var htmlDoc = document.popupNode.ownerDocument;
        var historyNode = htmlDoc.getElementById('inlinedefinitions-history');
        var historyEntry = historyNode.lastChild;
        var backPhrase = null;
        if (historyEntry) {
            backPhrase = historyEntry.getAttribute("inlinedefinitions-search-phrase");
        }
        
        if (backPhrase == null) {
            menuItem3.hidden = true;
        } else {
            menuItem3.hidden = false;
            menuItem3.setAttribute("label", 'Back to "' + backPhrase + '"');
        }

        /* if current phrase is selected, don't give option of searching again for it.. */
        var innerWindowContentBox = htmlDoc.getElementById('inlinedefinitions-content');
        var curPhrase = innerWindowContentBox.getAttribute("inlinedefinitions-search-phrase");
        if (selectedText != "" && selectedText == curPhrase) {
            menuItem1.hidden = true;
        }
        

    } else {
        menuItem2.hidden = menuItem3.hidden = true;
    }


    InlineDefinitionsExtension.popupLeft = event.pageX;
    InlineDefinitionsExtension.popupTop = event.pageY;
    InlineDefinitionsExtension.popupClientTop = event.clientY;
}

this.onMenuDefinitions = function (popup) { 
    var selection = InlineDefinitionsExtension.getSelection();
    var selectedText = selection.join(" ");   
    var url = "http://www.google.com/search?q=define%3A" + selection.join("+").toLowerCase();
    InlineDefinitionsExtension.log("onInlineDefinitions() : url = " + url);

    /* Check whether the popup was made from within the inline window - if so reuse it */
    var node = document.popupNode;
    var htmlDoc = document.popupNode.ownerDocument;

    if (InlineDefinitionsExtension.isInsideDefinitionsWindow(node)) {
        /* Save old content for 'back' button' */
        InlineDefinitionsExtension.log("loading into existing window (with back)");
        var innerWindowContentBox = htmlDoc.getElementById('inlinedefinitions-content');
        var historyNode = htmlDoc.getElementById('inlinedefinitions-history');
        var historyEntry = htmlDoc.createElement ( "div" )
        historyEntry.setAttribute( "inlinedefinitions-content", innerWindowContentBox.innerHTML );
        historyEntry.setAttribute( "inlinedefinitions-search-phrase", innerWindowContentBox.getAttribute("inlinedefinitions-search-phrase") );
        historyNode.appendChild(historyEntry);

        /* Show back button */
        var backButton = htmlDoc.getElementById('inlinedefinitions-back');
        backButton.style.display = "inline";

        InlineDefinitionsExtension.populate(url, htmlDoc, selectedText);

    } else if (InlineDefinitionsExtension.isWindowOpen(htmlDoc)) {
        /* clear back history and load into existing window so that it doesn't move */
        InlineDefinitionsExtension.log("loading into existing window (clearing back)");
        var innerWindowContentBox = htmlDoc.getElementById('inlinedefinitions-content');
        var historyNode = htmlDoc.getElementById('inlinedefinitions-history');
        while (historyNode.firstChild) {
            historyNode.removeChild(historyNode.firstChild);
         }

        /* hide back button */
        var backButton = htmlDoc.getElementById('inlinedefinitions-back');
        backButton.style.display = "none";

        InlineDefinitionsExtension.populate(url, htmlDoc, selectedText);

    } else {
        /* open new window */
        InlineDefinitionsExtension.log("creating new window");
        InlineDefinitionsExtension.newWindow(InlineDefinitionsExtension.popupLeft, InlineDefinitionsExtension.popupTop, htmlDoc, url, selectedText);
    }
}

this.onMenuClose = function (popup) { 
    var htmlDoc = document.popupNode.ownerDocument;
    InlineDefinitionsExtension.close(htmlDoc);
}

this.onMenuBack = function (popup) { 
    var htmlDoc = document.popupNode.ownerDocument;
    InlineDefinitionsExtension.back(htmlDoc);
}

this.isWindowOpen = function(htmlDoc) {
    var inlineWindow = htmlDoc.getElementById('inlinedefinitions-window');
    if ( inlineWindow ) {
        return true;
    } else {
        return false;
    }
}

this.isInsideDefinitionsWindow = function(node) {
    while (node) {
        
        InlineDefinitionsExtension.log(typeof(node));
        if (node.id == "inlinedefinitions-window") {
            return true;
        }
        node = node.parentNode;
    }
    return false;
}

this.getSelection = function() {
    
    var node = document.popupNode;
    var selection = "";

    if ((node instanceof HTMLTextAreaElement) || (node instanceof HTMLInputElement && node.type == "text")) {
        selection = node.value.substring(node.selectionStart, node.selectionEnd);
    } 
    else {
        var focusedWindow = new XPCNativeWrapper(document.commandDispatcher.focusedWindow, 'document', 'getSelection()');
        selection = focusedWindow.getSelection().toString();
    }
    //node.style.outline = "2px dashed blue !important"

    // Limit length to 150 to optimize performance. Longer does not make sense
    if (selection.length>=150){
        selection = selection.substring(0, 149);
    }
    // Strip newlines
    selection = selection.replace(/(\n|\r|\t)+/g, " ");
    // Strip spaces at start and end.
    selection = selection.replace(/(^\s+)|(\s+$)/g, "");

    selection = selection.split(" ");
   
    // Remove certain characters at the beginning and end of every word
    for (i=0; i<selection.length; i++){
        selection[i]=selection[i].replace(/^(\&|\(|\)|\[|\]|\{|\}|"|,|\.|!|\?|'|:|;)+/, "");
        selection[i]=selection[i].replace(/(\&|\(|\)|\[|\]|\{|\}|"|,|\.|!|\?|'|:|;)+$/, "");
    }   

    return selection; 
}

this.newWindow = function (left, top, htmlDoc, href, selectedText) {
    
    // Setup some constants for use in creating the inline window...
    var htmlWin = htmlDoc.defaultView;

    var windowWidth = Math.round(htmlDoc.width * 0.5);
    var windowHeight = Math.round(htmlWin.innerHeight * 0.5);
    var windowPadding = 13;
    var windowTextPadding = 5;
    var windowFontSize = 10;
    var windowBorderSize = 1;
    var windowButtonHeight = 12;
    var windowButtonTextSize = 11;
    var windowFullID = 'inlinedefinitions-window';
    
    // stop the window before getting this close to the left/right/top/bottom of the screen
    var pageBoundPadding = 10;
    
    var xpos, ypos;
    
    InlineDefinitionsExtension.log("popup at - (" + left + "," + top + ")");

    // setup the x-position of the inline window...
    // check to see if the left 1/3 of the window will overlap the left page bound..
    if ( left - (windowWidth/3) < pageBoundPadding ) {
        xpos = pageBoundPadding;
    } 
    // check to see if the right 2/3 of the window will overlap the right page bound...
    else if ( left + (windowWidth*2/3) > htmlDoc.width - pageBoundPadding ) {
        xpos = htmlDoc.width - pageBoundPadding - windowWidth;
    }
    else {
        // if we're not going to hit either wall, set the window to be offset
        // by 1/3 to the left of where we clicked (looks better than centering
        xpos = left - (windowWidth/3);
    }
    xpos = Math.round(xpos);
    
    // setup the y-positioning of the inline window...

    // check to see if the window goes beyond the bottom of the viewport...
    if ( InlineDefinitionsExtension.popupClientTop + windowHeight + pageBoundPadding > htmlWin.innerHeight ) {
        InlineDefinitionsExtension.log("goes below bottom");
        ypos = top - windowHeight + htmlWin.pageYOffset;
    } else { 
        InlineDefinitionsExtension.log("above bottom");
        ypos = top + htmlWin.pageYOffset;
    }

    var container = htmlDoc.createElement('div');
    container.id = windowFullID;
    
    var cssBoxWidth = Math.round((windowWidth - (windowPadding+windowBorderSize)*2)/htmlDoc.width*100);
    var cssBoxHeight = windowHeight - (windowPadding+windowBorderSize*2);

    InlineDefinitionsExtension.log("window height = " + windowHeight + "; boxHeight = " + cssBoxHeight + "; YOffset = " + htmlWin.pageYOffset);
    InlineDefinitionsExtension.log("showing window - (" + xpos + "," + ypos + ")");

    var margin = Math.round((windowPadding-windowButtonHeight)/2);

    container.innerHTML = '<div class="inlinedefinitions_draggable" style="' +
        'cursor: move; position: absolute; text-align: left; '+
        /* 'margin: ' + ypos + 'px 0 0 ' + xpos + 'px; ' + */
        'top: ' + ypos + 'px; left: ' + xpos + 'px; ' +
        'padding: ' + margin +'px ' + windowPadding + 'px ' + windowPadding + 'px; ' +
        'width: ' + cssBoxWidth + '%; ' +
        'height: '+ cssBoxHeight + 'px; background-color: #FFFFE0; color: black; '+
        'border: ' + windowBorderSize + 'px solid #E0E0E0; -moz-border-radius: 5px; z-index: 999; '+
        'font-size: ' + windowFontSize + 'pt;">'+
            '<div class="inlinedefinitions_draggable" style="'+
                '-moz-user-select: -moz-none; '+
                'float: left; cursor: move; '+
                'margin: 0 3px ' + margin + 'px; padding: 0 3px; -moz-border-radius: 2px; width: auto;' +
                'height: ' + windowButtonHeight + 'px; '+
                'font-size: ' + windowButtonTextSize + 'px; '+
                'line-height: ' + windowButtonTextSize + 'px; font-weight: bold'+
            '"><font color="#0039b6">G</font><font color="#c41200">o</font><font color="#f3c518">o</font><font color="#0039b6">g</font><font color="#30a72f">l</font><font color="#c41200">e</font> Definitions</div>'+
            '<div id="inlinedefinitions-close" style="'+
                'float: right; cursor: auto; background-color: #DDD; '+
                'margin: 0 3px ' + margin + 'px; '+
                'padding: 0 3px; -moz-border-radius: 2px; width: auto;' +
                'height: ' + windowButtonHeight + 'px; '+
                'font-size: ' + windowButtonTextSize + 'px; '+
                'line-height: ' + windowButtonTextSize + 'px; '+
                'font-weight: bold"><a href="#" style="text-decoration: none;">close</a></div>'+
            '<div id="inlinedefinitions-fullpage" style="'+
                'float: right; cursor: auto; background-color: #DDD; '+
                'margin: 0 3px ' + margin +'px; '+
                'padding: 0 3px; -moz-border-radius: 2px; width: auto;' +
                'height: ' + windowButtonHeight + 'px; '+
                'font-size: ' + windowButtonTextSize + 'px; '+
                'line-height: ' + windowButtonTextSize + 'px; '+
                'font-weight: bold"><a href="' + href + '" style="text-decoration: none;">full page</a></div>'+
            '<div id="inlinedefinitions-history" style="display: none; " ></div>' +
            '<div id="inlinedefinitions-back" style="'+
                'display: none; float: right; cursor: auto; background-color: #DDD; '+
                'margin: 0 3px ' + margin + 'px; '+
                'padding: 0 3px; -moz-border-radius: 2px; width: auto;' +
                'height: ' + windowButtonHeight + 'px; '+
                'font-size: ' + windowButtonTextSize + 'px; '+
                'line-height: ' + windowButtonTextSize + 'px; '+
                'font-weight: bold"><a href="#" style="text-decoration: none;">back</a></div>'+
            '<div id="inlinedefinitions-content" style="'+
                'cursor: auto; border: 1px dashed black; width: auto; background: white; padding: ' + windowTextPadding + 'px; '+
                'overflow: auto; clear: both; '+
                'height: ' + (windowHeight - (windowTextPadding+windowPadding+windowBorderSize)*2) + 'px; ' +
            '">loading<span style="text-decoration: blink">...</span></div>' 
            ''+ 
        '</div>';

    /* Delete any old windows and insert this one just after the <BODY> */
    InlineDefinitionsExtension.close(htmlDoc);
    htmlDoc.body.insertBefore(container, htmlDoc.body.firstChild);
    
    /* Register button events */
    var div = htmlDoc.getElementById("inlinedefinitions-close");
    div.firstChild.addEventListener("click", InlineDefinitionsExtension.onClose, false);
    div = htmlDoc.getElementById("inlinedefinitions-back");
    div.firstChild.addEventListener("click", InlineDefinitionsExtension.onBack, false);

    InlineDefinitionsDrag.init(container.firstChild);
    InlineDefinitionsExtension.populate(href, htmlDoc, selectedText);
}


this.populate = function (url, htmlDoc, selectedText) {
    
    var xmlhttp = new XMLHttpRequest();

    xmlhttp.onreadystatechange = function() {
            var headings, content;
            var innerWindowContentBox = htmlDoc.getElementById('inlinedefinitions-content');
            var xmlDoc;
            var found;

            if (xmlhttp.readyState != 4) /* wait for finished state */
                return;

            InlineDefinitionsExtension.log ("loaded! " + xmlhttp.status);

            if (xmlhttp.status != 200) {
                if (innerWindowContentBox) innerWindowContentBox.innerHTML = 'Error loading page.';
            } else {
            
                var definitions = null;

                if (xmlhttp.responseText.match("No definitions were found") != null) {
                    definitions = "No definitions were found";
                    found = false;
                } else {
                    /* log(xmlhttp.responseText); */
                    var re = new RegExp("<ul[\\s\\S]*</ul>");
                    var m = re.exec(xmlhttp.responseText);
                    if (m == null) InlineDefinitionsExtension.log("no match :!");
                    
                    if (m.length > 0) {
                        definitions = m[0];
                        found = true;
                    }
                    /* InlineDefinitionsExtension.log(definitions); */
                }
                
                if (innerWindowContentBox) {
                    innerWindowContentBox.setAttribute("inlinedefinitions-search-phrase", selectedText);
                    if (definitions) {
                        InlineDefinitionsExtension.log("loaded!");
                        innerWindowContentBox.innerHTML = definitions;
  
                        if (found) {
                            /* Fix links to sources */
                            source_links = innerWindowContentBox.getElementsByTagName("a");
                            var link,addr;
                            for (var i = 0; i < source_links.length; i++) {
                                link = source_links[i];
                                addr = link.getAttribute("href");
                                addr = "http://www.google.com" + addr;
                                link.setAttribute("href", addr);
                            }
                        }

                    } else {
                        InlineDefinitionsExtension.log("error");
                        innerWindowContentBox.innerHTML = 'Error loading page.';
                    }
                } else {
                    InlineDefinitionsExtension.log("Couldn't find a window '" + windowID + "' to populate content for '" + url + "'");
                }
            }
        };
    xmlhttp.open("GET", url, true);
    xmlhttp.send(null);
    InlineDefinitionsExtension.log("sending async request to + " + url);
}

this.onBack = function (e) { 
    InlineDefinitionsExtension.back(this.ownerDocument);
    e.preventDefault();
}

this.back = function (htmlDoc) {
    /* Save old content for 'back' button' */
    /* Restore old content */
    var innerWindowContentBox = htmlDoc.getElementById('inlinedefinitions-content');
    var historyNode = htmlDoc.getElementById('inlinedefinitions-history');
    var historyEntry = historyNode.lastChild;
    if (historyEntry) {
        innerWindowContentBox.innerHTML = historyEntry.getAttribute("inlinedefinitions-content");
        innerWindowContentBox.setAttribute( "inlinedefinitions-search-phrase", historyEntry.getAttribute("inlinedefinitions-search-phrase") );
        historyNode.removeChild(historyEntry);
    }

    /* Hide back button if no more history */
    if (! historyNode.hasChildNodes() ) {
        var backButton = htmlDoc.getElementById('inlinedefinitions-back');
        backButton.style.display = "none";
    }

}

this.onClose = function (e) {
    innerWindow = this.parentNode.parentNode.parentNode;
    innerWindow.parentNode.removeChild(innerWindow);
    e.preventDefault();
}

this.close = function (htmlDoc) { 
    var inlineWindow = htmlDoc.getElementById('inlinedefinitions-window');
    
    if ( inlineWindow ) {
        inlineWindow.style.display = 'none';
        inlineWindow.parentNode.removeChild(inlineWindow);
    }
    
}

/*
function getElementOffset(element,whichCoord) {
    var count = 0
    while (element!=null) {
        count += element['offset' + whichCoord];
        element = element.offsetParent;
    }
    return count;
}
*/

}).apply(InlineDefinitionsExtension);


window.addEventListener("load", InlineDefinitionsExtension.init,false);
